import base64
import json
import logging

from enum import Enum
from google.protobuf.timestamp_pb2 import Timestamp

import forge.utils as forge_utils
import event_chain.protos as protos

logger = logging.getLogger('ec-helpers')


def gen_timestamp(datetime):
    res = Timestamp()
    res.FromDatetime(datetime)
    return res


def add_to_proto_list(info, repeated):
    res = {item for item in repeated}
    res.add(info)
    return res


def remove_from_proto_list(info, repeated):
    res = filter(lambda item: item != info, repeated)
    return res


class WalletResponse:
    def __init__(self, response):
        self.response = response
        self.user_info = response.get('userInfo')
        self.decoded_info = self.decode_user_info()
        self.requested_claim = self.decoded_info.get('requestedClaims')[0]

    def decode_user_info(self):
        if not self.user_info:
            logger.error(
                "Fail to parse user_info from this Response {}.".format(
                    self.response,
                ),
            )
        else:
            sig = self.user_info.split('.')[1]
            decoded = base64.decodebytes(
                (sig + '=' * (-len(sig) % 4)).encode('utf8'),
            ).decode('utf8')
            dict = json.loads(decoded)
            logger.debug("User info is decoded successfully. {}".format(dict))
            return dict

    def get_address(self):
        did = self.decoded_info.get('iss')
        logger.debug("Parsed wallet address successfully!: {}".format(did))
        return did.split(':')[-1]

    def get_signature(self):
        sig = self.requested_claim.get('sig')
        logger.debug("Parsed wallet signature successfully! {}".format(sig))
        return sig

    def get_event_address(self):
        tx = self.requested_claim.get('tx')
        decoded_tx = base64.decodebytes((tx + '=' * (-len(tx) % 4)).encode('utf8'))

        parsed_tx = protos.Transaction()
        parsed_tx.ParseFromString(decoded_tx)
        logger.debug("Parsed tx successfully! {}".format(parsed_tx))
        exchange_itx = forge_utils.parse_to_proto(
            parsed_tx.itx.value,
            protos.ExchangeTx,
        )
        event_address = exchange_itx.data.value.decode('utf8')
        return event_address


def add_multi_sig_to_tx(tx, address, signature):
    new_tx = protos.Transaction()
    new_tx.CopyFrom(tx)
    multisig = protos.Multisig(
        signer=address,
        signature=signature.encode('utf8'),
    )
    new_tx.__setattr__('signatures', multisig)
    return new_tx


class ForgeTxType(Enum):
    ACTIVATE_ASSET = 'fg:t:activate_asset'
    CREATE_ASSET = 'fg:t:create_asset'
    EXCHANGE = 'fg:t:exchange'



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import abi_pb2 as abi__pb2


class ForgeAppRpcStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.process_one = channel.unary_unary(
        '/forge_abi.ForgeAppRpc/process_one',
        request_serializer=abi__pb2.Request.SerializeToString,
        response_deserializer=abi__pb2.Response.FromString,
        )
    self.process = channel.stream_stream(
        '/forge_abi.ForgeAppRpc/process',
        request_serializer=abi__pb2.Request.SerializeToString,
        response_deserializer=abi__pb2.Response.FromString,
        )


class ForgeAppRpcServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def process_one(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def process(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_ForgeAppRpcServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'process_one': grpc.unary_unary_rpc_method_handler(
          servicer.process_one,
          request_deserializer=abi__pb2.Request.FromString,
          response_serializer=abi__pb2.Response.SerializeToString,
      ),
      'process': grpc.stream_stream_rpc_method_handler(
          servicer.process,
          request_deserializer=abi__pb2.Request.FromString,
          response_serializer=abi__pb2.Response.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'forge_abi.ForgeAppRpc', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: trace_type.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import type_pb2 as type__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='trace_type.proto',
  package='forge_abi',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x10trace_type.proto\x12\tforge_abi\x1a\ntype.proto\"(\n\tPageOrder\x12\r\n\x05\x66ield\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\"N\n\tPageInput\x12\x0e\n\x06\x63ursor\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\r\x12#\n\x05order\x18\x03 \x03(\x0b\x32\x14.forge_abi.PageOrder\"\x1b\n\nTypeFilter\x12\r\n\x05types\x18\x01 \x03(\t\"<\n\nTimeFilter\x12\x17\n\x0fstart_date_time\x18\x01 \x01(\t\x12\x15\n\rend_date_time\x18\x02 \x01(\t\"Z\n\rAddressFilter\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x10\n\x08receiver\x18\x02 \x01(\t\x12\'\n\tdirection\x18\x03 \x01(\x0e\x32\x14.forge_abi.Direction\"7\n\x08PageInfo\x12\x0e\n\x06\x63ursor\x18\x01 \x01(\t\x12\x0c\n\x04next\x18\x02 \x01(\x08\x12\r\n\x05total\x18\x03 \x01(\r\"\x84\x01\n\x12IndexedTransaction\x12\x0c\n\x04hash\x18\x01 \x01(\t\x12\x0e\n\x06sender\x18\x02 \x01(\t\x12\x10\n\x08receiver\x18\x03 \x01(\t\x12\x0c\n\x04time\x18\x04 \x01(\t\x12\x0c\n\x04type\x18\x05 \x01(\t\x12\"\n\x02tx\x18\x06 \x01(\x0b\x32\x16.forge_abi.Transaction\"\x8d\x03\n\x13IndexedAccountState\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12#\n\x07\x62\x61lance\x18\x02 \x01(\x0b\x32\x12.forge_abi.BigUint\x12\x12\n\nnum_assets\x18\x03 \x01(\x04\x12\x0f\n\x07num_txs\x18\x04 \x01(\x04\x12\r\n\x05nonce\x18\x05 \x01(\x04\x12\x14\n\x0cgenesis_time\x18\x06 \x01(\t\x12\x18\n\x10renaissance_time\x18\x07 \x01(\t\x12\x0f\n\x07moniker\x18\x08 \x01(\t\x12\x15\n\rmigrated_from\x18\t \x01(\t\x12\x13\n\x0bmigrated_to\x18\n \x01(\t\x12\x31\n\x15total_received_stakes\x18\x0b \x01(\x0b\x32\x12.forge_abi.BigUint\x12(\n\x0ctotal_stakes\x18\x0c \x01(\x0b\x32\x12.forge_abi.BigUint\x12*\n\x0etotal_unstakes\x18\r \x01(\x0b\x32\x12.forge_abi.BigUint\x12\x16\n\x0erecent_num_txs\x18\x0e \x03(\x04\"\x86\x01\n\x11IndexedAssetState\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\r\n\x05owner\x18\x02 \x01(\t\x12\x14\n\x0cgenesis_time\x18\x03 \x01(\t\x12\x18\n\x10renaissance_time\x18\x04 \x01(\t\x12\x0f\n\x07moniker\x18\x05 \x01(\t\x12\x10\n\x08readonly\x18\x06 \x01(\x08\"\xba\x01\n\x11IndexedStakeState\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12#\n\x07\x62\x61lance\x18\x02 \x01(\x0b\x32\x12.forge_abi.BigUint\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x10\n\x08receiver\x18\x04 \x01(\t\x12\x14\n\x0cgenesis_time\x18\x05 \x01(\t\x12\x18\n\x10renaissance_time\x18\x06 \x01(\t\x12\x0f\n\x07message\x18\x07 \x01(\t\x12\x0c\n\x04type\x18\x08 \x01(\r*/\n\tDirection\x12\n\n\x06mutual\x10\x00\x12\x0b\n\x07one_way\x10\x01\x12\t\n\x05union\x10\x02\x62\x06proto3')
  ,
  dependencies=[type__pb2.DESCRIPTOR,])

_DIRECTION = _descriptor.EnumDescriptor(
  name='Direction',
  full_name='forge_abi.Direction',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='mutual', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='one_way', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='union', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1266,
  serialized_end=1313,
)
_sym_db.RegisterEnumDescriptor(_DIRECTION)

Direction = enum_type_wrapper.EnumTypeWrapper(_DIRECTION)
mutual = 0
one_way = 1
union = 2



_PAGEORDER = _descriptor.Descriptor(
  name='PageOrder',
  full_name='forge_abi.PageOrder',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='field', full_name='forge_abi.PageOrder.field', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='forge_abi.PageOrder.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=83,
)


_PAGEINPUT = _descriptor.Descriptor(
  name='PageInput',
  full_name='forge_abi.PageInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cursor', full_name='forge_abi.PageInput.cursor', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='forge_abi.PageInput.size', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='order', full_name='forge_abi.PageInput.order', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=85,
  serialized_end=163,
)


_TYPEFILTER = _descriptor.Descriptor(
  name='TypeFilter',
  full_name='forge_abi.TypeFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='types', full_name='forge_abi.TypeFilter.types', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=165,
  serialized_end=192,
)


_TIMEFILTER = _descriptor.Descriptor(
  name='TimeFilter',
  full_name='forge_abi.TimeFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_date_time', full_name='forge_abi.TimeFilter.start_date_time', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end_date_time', full_name='forge_abi.TimeFilter.end_date_time', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=194,
  serialized_end=254,
)


_ADDRESSFILTER = _descriptor.Descriptor(
  name='AddressFilter',
  full_name='forge_abi.AddressFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender', full_name='forge_abi.AddressFilter.sender', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiver', full_name='forge_abi.AddressFilter.receiver', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='direction', full_name='forge_abi.AddressFilter.direction', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=256,
  serialized_end=346,
)


_PAGEINFO = _descriptor.Descriptor(
  name='PageInfo',
  full_name='forge_abi.PageInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cursor', full_name='forge_abi.PageInfo.cursor', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='next', full_name='forge_abi.PageInfo.next', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total', full_name='forge_abi.PageInfo.total', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=348,
  serialized_end=403,
)


_INDEXEDTRANSACTION = _descriptor.Descriptor(
  name='IndexedTransaction',
  full_name='forge_abi.IndexedTransaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='forge_abi.IndexedTransaction.hash', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sender', full_name='forge_abi.IndexedTransaction.sender', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiver', full_name='forge_abi.IndexedTransaction.receiver', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='forge_abi.IndexedTransaction.time', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='forge_abi.IndexedTransaction.type', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tx', full_name='forge_abi.IndexedTransaction.tx', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=406,
  serialized_end=538,
)


_INDEXEDACCOUNTSTATE = _descriptor.Descriptor(
  name='IndexedAccountState',
  full_name='forge_abi.IndexedAccountState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='forge_abi.IndexedAccountState.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='balance', full_name='forge_abi.IndexedAccountState.balance', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_assets', full_name='forge_abi.IndexedAccountState.num_assets', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_txs', full_name='forge_abi.IndexedAccountState.num_txs', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nonce', full_name='forge_abi.IndexedAccountState.nonce', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genesis_time', full_name='forge_abi.IndexedAccountState.genesis_time', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='renaissance_time', full_name='forge_abi.IndexedAccountState.renaissance_time', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='moniker', full_name='forge_abi.IndexedAccountState.moniker', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='migrated_from', full_name='forge_abi.IndexedAccountState.migrated_from', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='migrated_to', full_name='forge_abi.IndexedAccountState.migrated_to', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_received_stakes', full_name='forge_abi.IndexedAccountState.total_received_stakes', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_stakes', full_name='forge_abi.IndexedAccountState.total_stakes', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_unstakes', full_name='forge_abi.IndexedAccountState.total_unstakes', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recent_num_txs', full_name='forge_abi.IndexedAccountState.recent_num_txs', index=13,
      number=14, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=541,
  serialized_end=938,
)


_INDEXEDASSETSTATE = _descriptor.Descriptor(
  name='IndexedAssetState',
  full_name='forge_abi.IndexedAssetState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='forge_abi.IndexedAssetState.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='owner', full_name='forge_abi.IndexedAssetState.owner', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genesis_time', full_name='forge_abi.IndexedAssetState.genesis_time', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='renaissance_time', full_name='forge_abi.IndexedAssetState.renaissance_time', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='moniker', full_name='forge_abi.IndexedAssetState.moniker', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='readonly', full_name='forge_abi.IndexedAssetState.readonly', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=941,
  serialized_end=1075,
)


_INDEXEDSTAKESTATE = _descriptor.Descriptor(
  name='IndexedStakeState',
  full_name='forge_abi.IndexedStakeState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='forge_abi.IndexedStakeState.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='balance', full_name='forge_abi.IndexedStakeState.balance', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sender', full_name='forge_abi.IndexedStakeState.sender', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiver', full_name='forge_abi.IndexedStakeState.receiver', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genesis_time', full_name='forge_abi.IndexedStakeState.genesis_time', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='renaissance_time', full_name='forge_abi.IndexedStakeState.renaissance_time', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='forge_abi.IndexedStakeState.message', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='forge_abi.IndexedStakeState.type', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1078,
  serialized_end=1264,
)

_PAGEINPUT.fields_by_name['order'].message_type = _PAGEORDER
_ADDRESSFILTER.fields_by_name['direction'].enum_type = _DIRECTION
_INDEXEDTRANSACTION.fields_by_name['tx'].message_type = type__pb2._TRANSACTION
_INDEXEDACCOUNTSTATE.fields_by_name['balance'].message_type = type__pb2._BIGUINT
_INDEXEDACCOUNTSTATE.fields_by_name['total_received_stakes'].message_type = type__pb2._BIGUINT
_INDEXEDACCOUNTSTATE.fields_by_name['total_stakes'].message_type = type__pb2._BIGUINT
_INDEXEDACCOUNTSTATE.fields_by_name['total_unstakes'].message_type = type__pb2._BIGUINT
_INDEXEDSTAKESTATE.fields_by_name['balance'].message_type = type__pb2._BIGUINT
DESCRIPTOR.message_types_by_name['PageOrder'] = _PAGEORDER
DESCRIPTOR.message_types_by_name['PageInput'] = _PAGEINPUT
DESCRIPTOR.message_types_by_name['TypeFilter'] = _TYPEFILTER
DESCRIPTOR.message_types_by_name['TimeFilter'] = _TIMEFILTER
DESCRIPTOR.message_types_by_name['AddressFilter'] = _ADDRESSFILTER
DESCRIPTOR.message_types_by_name['PageInfo'] = _PAGEINFO
DESCRIPTOR.message_types_by_name['IndexedTransaction'] = _INDEXEDTRANSACTION
DESCRIPTOR.message_types_by_name['IndexedAccountState'] = _INDEXEDACCOUNTSTATE
DESCRIPTOR.message_types_by_name['IndexedAssetState'] = _INDEXEDASSETSTATE
DESCRIPTOR.message_types_by_name['IndexedStakeState'] = _INDEXEDSTAKESTATE
DESCRIPTOR.enum_types_by_name['Direction'] = _DIRECTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PageOrder = _reflection.GeneratedProtocolMessageType('PageOrder', (_message.Message,), dict(
  DESCRIPTOR = _PAGEORDER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.PageOrder)
  ))
_sym_db.RegisterMessage(PageOrder)

PageInput = _reflection.GeneratedProtocolMessageType('PageInput', (_message.Message,), dict(
  DESCRIPTOR = _PAGEINPUT,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.PageInput)
  ))
_sym_db.RegisterMessage(PageInput)

TypeFilter = _reflection.GeneratedProtocolMessageType('TypeFilter', (_message.Message,), dict(
  DESCRIPTOR = _TYPEFILTER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.TypeFilter)
  ))
_sym_db.RegisterMessage(TypeFilter)

TimeFilter = _reflection.GeneratedProtocolMessageType('TimeFilter', (_message.Message,), dict(
  DESCRIPTOR = _TIMEFILTER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.TimeFilter)
  ))
_sym_db.RegisterMessage(TimeFilter)

AddressFilter = _reflection.GeneratedProtocolMessageType('AddressFilter', (_message.Message,), dict(
  DESCRIPTOR = _ADDRESSFILTER,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.AddressFilter)
  ))
_sym_db.RegisterMessage(AddressFilter)

PageInfo = _reflection.GeneratedProtocolMessageType('PageInfo', (_message.Message,), dict(
  DESCRIPTOR = _PAGEINFO,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.PageInfo)
  ))
_sym_db.RegisterMessage(PageInfo)

IndexedTransaction = _reflection.GeneratedProtocolMessageType('IndexedTransaction', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDTRANSACTION,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedTransaction)
  ))
_sym_db.RegisterMessage(IndexedTransaction)

IndexedAccountState = _reflection.GeneratedProtocolMessageType('IndexedAccountState', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDACCOUNTSTATE,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedAccountState)
  ))
_sym_db.RegisterMessage(IndexedAccountState)

IndexedAssetState = _reflection.GeneratedProtocolMessageType('IndexedAssetState', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDASSETSTATE,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedAssetState)
  ))
_sym_db.RegisterMessage(IndexedAssetState)

IndexedStakeState = _reflection.GeneratedProtocolMessageType('IndexedStakeState', (_message.Message,), dict(
  DESCRIPTOR = _INDEXEDSTAKESTATE,
  __module__ = 'trace_type_pb2'
  # @@protoc_insertion_point(class_scope:forge_abi.IndexedStakeState)
  ))
_sym_db.RegisterMessage(IndexedStakeState)


# @@protoc_insertion_point(module_scope)



from event_chain.config import config
import logging
from event_chain.db import utils
logger = logging.getLogger('ec-db-init')

if __name__ == '__main__':
    db = config.db_path
    logger.info('Connecting to db {}'.format(db))
    conn = utils.create_connection()
    utils.init_db(conn)
    conn.close()




# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import rpc_pb2 as rpc__pb2


class ChainRpcStub(object):
  """forge RPC definition

  Notice: when you define a new RPC, please follow the naming convention. Your
  function name is snake case, and req / req are PASCAL case of the function
  name, prefixed with Request / Response. e.g. rpc get_abc(RequestGetAbc)
  returns (ResponseGetAbc). If you break this, RPC builder would complain.

  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.create_tx = channel.unary_unary(
        '/forge_abi.ChainRpc/create_tx',
        request_serializer=rpc__pb2.RequestCreateTx.SerializeToString,
        response_deserializer=rpc__pb2.ResponseCreateTx.FromString,
        )
    self.multisig = channel.unary_unary(
        '/forge_abi.ChainRpc/multisig',
        request_serializer=rpc__pb2.RequestMultisig.SerializeToString,
        response_deserializer=rpc__pb2.ResponseMultisig.FromString,
        )
    self.send_tx = channel.unary_unary(
        '/forge_abi.ChainRpc/send_tx',
        request_serializer=rpc__pb2.RequestSendTx.SerializeToString,
        response_deserializer=rpc__pb2.ResponseSendTx.FromString,
        )
    self.get_tx = channel.stream_stream(
        '/forge_abi.ChainRpc/get_tx',
        request_serializer=rpc__pb2.RequestGetTx.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetTx.FromString,
        )
    self.get_block = channel.stream_stream(
        '/forge_abi.ChainRpc/get_block',
        request_serializer=rpc__pb2.RequestGetBlock.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetBlock.FromString,
        )
    self.get_blocks = channel.unary_unary(
        '/forge_abi.ChainRpc/get_blocks',
        request_serializer=rpc__pb2.RequestGetBlocks.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetBlocks.FromString,
        )
    self.get_unconfirmed_txs = channel.unary_unary(
        '/forge_abi.ChainRpc/get_unconfirmed_txs',
        request_serializer=rpc__pb2.RequestGetUnconfirmedTxs.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetUnconfirmedTxs.FromString,
        )
    self.get_chain_info = channel.unary_unary(
        '/forge_abi.ChainRpc/get_chain_info',
        request_serializer=rpc__pb2.RequestGetChainInfo.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetChainInfo.FromString,
        )
    self.get_node_info = channel.unary_unary(
        '/forge_abi.ChainRpc/get_node_info',
        request_serializer=rpc__pb2.RequestGetNodeInfo.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetNodeInfo.FromString,
        )
    self.search = channel.unary_unary(
        '/forge_abi.ChainRpc/search',
        request_serializer=rpc__pb2.RequestSearch.SerializeToString,
        response_deserializer=rpc__pb2.ResponseSearch.FromString,
        )
    self.get_net_info = channel.unary_unary(
        '/forge_abi.ChainRpc/get_net_info',
        request_serializer=rpc__pb2.RequestGetNetInfo.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetNetInfo.FromString,
        )
    self.get_validators_info = channel.unary_unary(
        '/forge_abi.ChainRpc/get_validators_info',
        request_serializer=rpc__pb2.RequestGetValidatorsInfo.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetValidatorsInfo.FromString,
        )
    self.get_config = channel.unary_unary(
        '/forge_abi.ChainRpc/get_config',
        request_serializer=rpc__pb2.RequestGetConfig.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetConfig.FromString,
        )
    self.get_asset_address = channel.unary_unary(
        '/forge_abi.ChainRpc/get_asset_address',
        request_serializer=rpc__pb2.RequestGetAssetAddress.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetAssetAddress.FromString,
        )
    self.sign_data = channel.unary_unary(
        '/forge_abi.ChainRpc/sign_data',
        request_serializer=rpc__pb2.RequestSignData.SerializeToString,
        response_deserializer=rpc__pb2.ResponseSignData.FromString,
        )


class ChainRpcServicer(object):
  """forge RPC definition

  Notice: when you define a new RPC, please follow the naming convention. Your
  function name is snake case, and req / req are PASCAL case of the function
  name, prefixed with Request / Response. e.g. rpc get_abc(RequestGetAbc)
  returns (ResponseGetAbc). If you break this, RPC builder would complain.

  """

  def create_tx(self, request, context):
    """tx related
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def multisig(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def send_tx(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_tx(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_block(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_blocks(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_unconfirmed_txs(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_chain_info(self, request, context):
    """utility
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_node_info(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def search(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_net_info(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_validators_info(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_config(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_asset_address(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def sign_data(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_ChainRpcServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'create_tx': grpc.unary_unary_rpc_method_handler(
          servicer.create_tx,
          request_deserializer=rpc__pb2.RequestCreateTx.FromString,
          response_serializer=rpc__pb2.ResponseCreateTx.SerializeToString,
      ),
      'multisig': grpc.unary_unary_rpc_method_handler(
          servicer.multisig,
          request_deserializer=rpc__pb2.RequestMultisig.FromString,
          response_serializer=rpc__pb2.ResponseMultisig.SerializeToString,
      ),
      'send_tx': grpc.unary_unary_rpc_method_handler(
          servicer.send_tx,
          request_deserializer=rpc__pb2.RequestSendTx.FromString,
          response_serializer=rpc__pb2.ResponseSendTx.SerializeToString,
      ),
      'get_tx': grpc.stream_stream_rpc_method_handler(
          servicer.get_tx,
          request_deserializer=rpc__pb2.RequestGetTx.FromString,
          response_serializer=rpc__pb2.ResponseGetTx.SerializeToString,
      ),
      'get_block': grpc.stream_stream_rpc_method_handler(
          servicer.get_block,
          request_deserializer=rpc__pb2.RequestGetBlock.FromString,
          response_serializer=rpc__pb2.ResponseGetBlock.SerializeToString,
      ),
      'get_blocks': grpc.unary_unary_rpc_method_handler(
          servicer.get_blocks,
          request_deserializer=rpc__pb2.RequestGetBlocks.FromString,
          response_serializer=rpc__pb2.ResponseGetBlocks.SerializeToString,
      ),
      'get_unconfirmed_txs': grpc.unary_unary_rpc_method_handler(
          servicer.get_unconfirmed_txs,
          request_deserializer=rpc__pb2.RequestGetUnconfirmedTxs.FromString,
          response_serializer=rpc__pb2.ResponseGetUnconfirmedTxs.SerializeToString,
      ),
      'get_chain_info': grpc.unary_unary_rpc_method_handler(
          servicer.get_chain_info,
          request_deserializer=rpc__pb2.RequestGetChainInfo.FromString,
          response_serializer=rpc__pb2.ResponseGetChainInfo.SerializeToString,
      ),
      'get_node_info': grpc.unary_unary_rpc_method_handler(
          servicer.get_node_info,
          request_deserializer=rpc__pb2.RequestGetNodeInfo.FromString,
          response_serializer=rpc__pb2.ResponseGetNodeInfo.SerializeToString,
      ),
      'search': grpc.unary_unary_rpc_method_handler(
          servicer.search,
          request_deserializer=rpc__pb2.RequestSearch.FromString,
          response_serializer=rpc__pb2.ResponseSearch.SerializeToString,
      ),
      'get_net_info': grpc.unary_unary_rpc_method_handler(
          servicer.get_net_info,
          request_deserializer=rpc__pb2.RequestGetNetInfo.FromString,
          response_serializer=rpc__pb2.ResponseGetNetInfo.SerializeToString,
      ),
      'get_validators_info': grpc.unary_unary_rpc_method_handler(
          servicer.get_validators_info,
          request_deserializer=rpc__pb2.RequestGetValidatorsInfo.FromString,
          response_serializer=rpc__pb2.ResponseGetValidatorsInfo.SerializeToString,
      ),
      'get_config': grpc.unary_unary_rpc_method_handler(
          servicer.get_config,
          request_deserializer=rpc__pb2.RequestGetConfig.FromString,
          response_serializer=rpc__pb2.ResponseGetConfig.SerializeToString,
      ),
      'get_asset_address': grpc.unary_unary_rpc_method_handler(
          servicer.get_asset_address,
          request_deserializer=rpc__pb2.RequestGetAssetAddress.FromString,
          response_serializer=rpc__pb2.ResponseGetAssetAddress.SerializeToString,
      ),
      'sign_data': grpc.unary_unary_rpc_method_handler(
          servicer.sign_data,
          request_deserializer=rpc__pb2.RequestSignData.FromString,
          response_serializer=rpc__pb2.ResponseSignData.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'forge_abi.ChainRpc', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class EventRpcStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.subscribe = channel.unary_stream(
        '/forge_abi.EventRpc/subscribe',
        request_serializer=rpc__pb2.RequestSubscribe.SerializeToString,
        response_deserializer=rpc__pb2.ResponseSubscribe.FromString,
        )
    self.unsubscribe = channel.unary_unary(
        '/forge_abi.EventRpc/unsubscribe',
        request_serializer=rpc__pb2.RequestUnsubscribe.SerializeToString,
        response_deserializer=rpc__pb2.ResponseUnsubscribe.FromString,
        )


class EventRpcServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def subscribe(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def unsubscribe(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_EventRpcServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'subscribe': grpc.unary_stream_rpc_method_handler(
          servicer.subscribe,
          request_deserializer=rpc__pb2.RequestSubscribe.FromString,
          response_serializer=rpc__pb2.ResponseSubscribe.SerializeToString,
      ),
      'unsubscribe': grpc.unary_unary_rpc_method_handler(
          servicer.unsubscribe,
          request_deserializer=rpc__pb2.RequestUnsubscribe.FromString,
          response_serializer=rpc__pb2.ResponseUnsubscribe.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'forge_abi.EventRpc', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class FileRpcStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.store_file = channel.stream_unary(
        '/forge_abi.FileRpc/store_file',
        request_serializer=rpc__pb2.RequestStoreFile.SerializeToString,
        response_deserializer=rpc__pb2.ResponseStoreFile.FromString,
        )
    self.load_file = channel.unary_stream(
        '/forge_abi.FileRpc/load_file',
        request_serializer=rpc__pb2.RequestLoadFile.SerializeToString,
        response_deserializer=rpc__pb2.ResponseLoadFile.FromString,
        )
    self.pin_file = channel.unary_unary(
        '/forge_abi.FileRpc/pin_file',
        request_serializer=rpc__pb2.RequestPinFile.SerializeToString,
        response_deserializer=rpc__pb2.ResponsePinFile.FromString,
        )


class FileRpcServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def store_file(self, request_iterator, context):
    """filesystem related
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def load_file(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def pin_file(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_FileRpcServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'store_file': grpc.stream_unary_rpc_method_handler(
          servicer.store_file,
          request_deserializer=rpc__pb2.RequestStoreFile.FromString,
          response_serializer=rpc__pb2.ResponseStoreFile.SerializeToString,
      ),
      'load_file': grpc.unary_stream_rpc_method_handler(
          servicer.load_file,
          request_deserializer=rpc__pb2.RequestLoadFile.FromString,
          response_serializer=rpc__pb2.ResponseLoadFile.SerializeToString,
      ),
      'pin_file': grpc.unary_unary_rpc_method_handler(
          servicer.pin_file,
          request_deserializer=rpc__pb2.RequestPinFile.FromString,
          response_serializer=rpc__pb2.ResponsePinFile.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'forge_abi.FileRpc', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class StateRpcStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.get_account_state = channel.stream_stream(
        '/forge_abi.StateRpc/get_account_state',
        request_serializer=rpc__pb2.RequestGetAccountState.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetAccountState.FromString,
        )
    self.get_asset_state = channel.stream_stream(
        '/forge_abi.StateRpc/get_asset_state',
        request_serializer=rpc__pb2.RequestGetAssetState.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetAssetState.FromString,
        )
    self.get_stake_state = channel.stream_stream(
        '/forge_abi.StateRpc/get_stake_state',
        request_serializer=rpc__pb2.RequestGetStakeState.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetStakeState.FromString,
        )
    self.get_forge_state = channel.unary_unary(
        '/forge_abi.StateRpc/get_forge_state',
        request_serializer=rpc__pb2.RequestGetForgeState.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetForgeState.FromString,
        )


class StateRpcServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def get_account_state(self, request_iterator, context):
    """state related
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_asset_state(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_stake_state(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_forge_state(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_StateRpcServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'get_account_state': grpc.stream_stream_rpc_method_handler(
          servicer.get_account_state,
          request_deserializer=rpc__pb2.RequestGetAccountState.FromString,
          response_serializer=rpc__pb2.ResponseGetAccountState.SerializeToString,
      ),
      'get_asset_state': grpc.stream_stream_rpc_method_handler(
          servicer.get_asset_state,
          request_deserializer=rpc__pb2.RequestGetAssetState.FromString,
          response_serializer=rpc__pb2.ResponseGetAssetState.SerializeToString,
      ),
      'get_stake_state': grpc.stream_stream_rpc_method_handler(
          servicer.get_stake_state,
          request_deserializer=rpc__pb2.RequestGetStakeState.FromString,
          response_serializer=rpc__pb2.ResponseGetStakeState.SerializeToString,
      ),
      'get_forge_state': grpc.unary_unary_rpc_method_handler(
          servicer.get_forge_state,
          request_deserializer=rpc__pb2.RequestGetForgeState.FromString,
          response_serializer=rpc__pb2.ResponseGetForgeState.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'forge_abi.StateRpc', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class WalletRpcStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.create_wallet = channel.unary_unary(
        '/forge_abi.WalletRpc/create_wallet',
        request_serializer=rpc__pb2.RequestCreateWallet.SerializeToString,
        response_deserializer=rpc__pb2.ResponseCreateWallet.FromString,
        )
    self.load_wallet = channel.unary_unary(
        '/forge_abi.WalletRpc/load_wallet',
        request_serializer=rpc__pb2.RequestLoadWallet.SerializeToString,
        response_deserializer=rpc__pb2.ResponseLoadWallet.FromString,
        )
    self.recover_wallet = channel.unary_unary(
        '/forge_abi.WalletRpc/recover_wallet',
        request_serializer=rpc__pb2.RequestRecoverWallet.SerializeToString,
        response_deserializer=rpc__pb2.ResponseRecoverWallet.FromString,
        )
    self.list_wallet = channel.unary_stream(
        '/forge_abi.WalletRpc/list_wallet',
        request_serializer=rpc__pb2.RequestListWallet.SerializeToString,
        response_deserializer=rpc__pb2.ResponseListWallet.FromString,
        )
    self.remove_wallet = channel.unary_unary(
        '/forge_abi.WalletRpc/remove_wallet',
        request_serializer=rpc__pb2.RequestRemoveWallet.SerializeToString,
        response_deserializer=rpc__pb2.ResponseRemoveWallet.FromString,
        )
    self.declare_node = channel.unary_unary(
        '/forge_abi.WalletRpc/declare_node',
        request_serializer=rpc__pb2.RequestDeclareNode.SerializeToString,
        response_deserializer=rpc__pb2.ResponseDeclareNode.FromString,
        )


class WalletRpcServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def create_wallet(self, request, context):
    """wallet related
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def load_wallet(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def recover_wallet(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def list_wallet(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def remove_wallet(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def declare_node(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_WalletRpcServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'create_wallet': grpc.unary_unary_rpc_method_handler(
          servicer.create_wallet,
          request_deserializer=rpc__pb2.RequestCreateWallet.FromString,
          response_serializer=rpc__pb2.ResponseCreateWallet.SerializeToString,
      ),
      'load_wallet': grpc.unary_unary_rpc_method_handler(
          servicer.load_wallet,
          request_deserializer=rpc__pb2.RequestLoadWallet.FromString,
          response_serializer=rpc__pb2.ResponseLoadWallet.SerializeToString,
      ),
      'recover_wallet': grpc.unary_unary_rpc_method_handler(
          servicer.recover_wallet,
          request_deserializer=rpc__pb2.RequestRecoverWallet.FromString,
          response_serializer=rpc__pb2.ResponseRecoverWallet.SerializeToString,
      ),
      'list_wallet': grpc.unary_stream_rpc_method_handler(
          servicer.list_wallet,
          request_deserializer=rpc__pb2.RequestListWallet.FromString,
          response_serializer=rpc__pb2.ResponseListWallet.SerializeToString,
      ),
      'remove_wallet': grpc.unary_unary_rpc_method_handler(
          servicer.remove_wallet,
          request_deserializer=rpc__pb2.RequestRemoveWallet.FromString,
          response_serializer=rpc__pb2.ResponseRemoveWallet.SerializeToString,
      ),
      'declare_node': grpc.unary_unary_rpc_method_handler(
          servicer.declare_node,
          request_deserializer=rpc__pb2.RequestDeclareNode.FromString,
          response_serializer=rpc__pb2.ResponseDeclareNode.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'forge_abi.WalletRpc', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class StatisticRpcStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.get_forge_statistics = channel.unary_unary(
        '/forge_abi.StatisticRpc/get_forge_statistics',
        request_serializer=rpc__pb2.RequestGetForgeStatistics.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetForgeStatistics.FromString,
        )
    self.list_transactions = channel.unary_unary(
        '/forge_abi.StatisticRpc/list_transactions',
        request_serializer=rpc__pb2.RequestListTransactions.SerializeToString,
        response_deserializer=rpc__pb2.ResponseListTransactions.FromString,
        )
    self.get_assets = channel.unary_unary(
        '/forge_abi.StatisticRpc/get_assets',
        request_serializer=rpc__pb2.RequestGetAssets.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetAssets.FromString,
        )
    self.get_stakes = channel.unary_unary(
        '/forge_abi.StatisticRpc/get_stakes',
        request_serializer=rpc__pb2.RequestGetStakes.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetStakes.FromString,
        )
    self.get_top_accounts = channel.unary_unary(
        '/forge_abi.StatisticRpc/get_top_accounts',
        request_serializer=rpc__pb2.RequestGetTopAccounts.SerializeToString,
        response_deserializer=rpc__pb2.ResponseGetTopAccounts.FromString,
        )
    self.list_asset_transactions = channel.unary_unary(
        '/forge_abi.StatisticRpc/list_asset_transactions',
        request_serializer=rpc__pb2.RequestListAssetTransactions.SerializeToString,
        response_deserializer=rpc__pb2.ResponseListAssetTransactions.FromString,
        )


class StatisticRpcServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def get_forge_statistics(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def list_transactions(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_assets(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_stakes(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_top_accounts(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def list_asset_transactions(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_StatisticRpcServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'get_forge_statistics': grpc.unary_unary_rpc_method_handler(
          servicer.get_forge_statistics,
          request_deserializer=rpc__pb2.RequestGetForgeStatistics.FromString,
          response_serializer=rpc__pb2.ResponseGetForgeStatistics.SerializeToString,
      ),
      'list_transactions': grpc.unary_unary_rpc_method_handler(
          servicer.list_transactions,
          request_deserializer=rpc__pb2.RequestListTransactions.FromString,
          response_serializer=rpc__pb2.ResponseListTransactions.SerializeToString,
      ),
      'get_assets': grpc.unary_unary_rpc_method_handler(
          servicer.get_assets,
          request_deserializer=rpc__pb2.RequestGetAssets.FromString,
          response_serializer=rpc__pb2.ResponseGetAssets.SerializeToString,
      ),
      'get_stakes': grpc.unary_unary_rpc_method_handler(
          servicer.get_stakes,
          request_deserializer=rpc__pb2.RequestGetStakes.FromString,
          response_serializer=rpc__pb2.ResponseGetStakes.SerializeToString,
      ),
      'get_top_accounts': grpc.unary_unary_rpc_method_handler(
          servicer.get_top_accounts,
          request_deserializer=rpc__pb2.RequestGetTopAccounts.FromString,
          response_serializer=rpc__pb2.ResponseGetTopAccounts.SerializeToString,
      ),
      'list_asset_transactions': grpc.unary_unary_rpc_method_handler(
          servicer.list_asset_transactions,
          request_deserializer=rpc__pb2.RequestListAssetTransactions.FromString,
          response_serializer=rpc__pb2.ResponseListAssetTransactions.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'forge_abi.StatisticRpc', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import vendor_pb2 as vendor__pb2


class ABCIApplicationStub(object):
  """----------------------------------------
  Service Definition

  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Echo = channel.unary_unary(
        '/abci_vendor.ABCIApplication/Echo',
        request_serializer=vendor__pb2.RequestEcho.SerializeToString,
        response_deserializer=vendor__pb2.ResponseEcho.FromString,
        )
    self.Flush = channel.unary_unary(
        '/abci_vendor.ABCIApplication/Flush',
        request_serializer=vendor__pb2.RequestFlush.SerializeToString,
        response_deserializer=vendor__pb2.ResponseFlush.FromString,
        )
    self.Info = channel.unary_unary(
        '/abci_vendor.ABCIApplication/Info',
        request_serializer=vendor__pb2.RequestInfo.SerializeToString,
        response_deserializer=vendor__pb2.ResponseInfo.FromString,
        )
    self.SetOption = channel.unary_unary(
        '/abci_vendor.ABCIApplication/SetOption',
        request_serializer=vendor__pb2.RequestSetOption.SerializeToString,
        response_deserializer=vendor__pb2.ResponseSetOption.FromString,
        )
    self.DeliverTx = channel.unary_unary(
        '/abci_vendor.ABCIApplication/DeliverTx',
        request_serializer=vendor__pb2.RequestDeliverTx.SerializeToString,
        response_deserializer=vendor__pb2.ResponseDeliverTx.FromString,
        )
    self.CheckTx = channel.unary_unary(
        '/abci_vendor.ABCIApplication/CheckTx',
        request_serializer=vendor__pb2.RequestCheckTx.SerializeToString,
        response_deserializer=vendor__pb2.ResponseCheckTx.FromString,
        )
    self.Query = channel.unary_unary(
        '/abci_vendor.ABCIApplication/Query',
        request_serializer=vendor__pb2.RequestQuery.SerializeToString,
        response_deserializer=vendor__pb2.ResponseQuery.FromString,
        )
    self.Commit = channel.unary_unary(
        '/abci_vendor.ABCIApplication/Commit',
        request_serializer=vendor__pb2.RequestCommit.SerializeToString,
        response_deserializer=vendor__pb2.ResponseCommit.FromString,
        )
    self.InitChain = channel.unary_unary(
        '/abci_vendor.ABCIApplication/InitChain',
        request_serializer=vendor__pb2.RequestInitChain.SerializeToString,
        response_deserializer=vendor__pb2.ResponseInitChain.FromString,
        )
    self.BeginBlock = channel.unary_unary(
        '/abci_vendor.ABCIApplication/BeginBlock',
        request_serializer=vendor__pb2.RequestBeginBlock.SerializeToString,
        response_deserializer=vendor__pb2.ResponseBeginBlock.FromString,
        )
    self.EndBlock = channel.unary_unary(
        '/abci_vendor.ABCIApplication/EndBlock',
        request_serializer=vendor__pb2.RequestEndBlock.SerializeToString,
        response_deserializer=vendor__pb2.ResponseEndBlock.FromString,
        )


class ABCIApplicationServicer(object):
  """----------------------------------------
  Service Definition

  """

  def Echo(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Flush(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Info(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SetOption(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeliverTx(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CheckTx(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Query(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Commit(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def InitChain(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def BeginBlock(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def EndBlock(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_ABCIApplicationServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Echo': grpc.unary_unary_rpc_method_handler(
          servicer.Echo,
          request_deserializer=vendor__pb2.RequestEcho.FromString,
          response_serializer=vendor__pb2.ResponseEcho.SerializeToString,
      ),
      'Flush': grpc.unary_unary_rpc_method_handler(
          servicer.Flush,
          request_deserializer=vendor__pb2.RequestFlush.FromString,
          response_serializer=vendor__pb2.ResponseFlush.SerializeToString,
      ),
      'Info': grpc.unary_unary_rpc_method_handler(
          servicer.Info,
          request_deserializer=vendor__pb2.RequestInfo.FromString,
          response_serializer=vendor__pb2.ResponseInfo.SerializeToString,
      ),
      'SetOption': grpc.unary_unary_rpc_method_handler(
          servicer.SetOption,
          request_deserializer=vendor__pb2.RequestSetOption.FromString,
          response_serializer=vendor__pb2.ResponseSetOption.SerializeToString,
      ),
      'DeliverTx': grpc.unary_unary_rpc_method_handler(
          servicer.DeliverTx,
          request_deserializer=vendor__pb2.RequestDeliverTx.FromString,
          response_serializer=vendor__pb2.ResponseDeliverTx.SerializeToString,
      ),
      'CheckTx': grpc.unary_unary_rpc_method_handler(
          servicer.CheckTx,
          request_deserializer=vendor__pb2.RequestCheckTx.FromString,
          response_serializer=vendor__pb2.ResponseCheckTx.SerializeToString,
      ),
      'Query': grpc.unary_unary_rpc_method_handler(
          servicer.Query,
          request_deserializer=vendor__pb2.RequestQuery.FromString,
          response_serializer=vendor__pb2.ResponseQuery.SerializeToString,
      ),
      'Commit': grpc.unary_unary_rpc_method_handler(
          servicer.Commit,
          request_deserializer=vendor__pb2.RequestCommit.FromString,
          response_serializer=vendor__pb2.ResponseCommit.SerializeToString,
      ),
      'InitChain': grpc.unary_unary_rpc_method_handler(
          servicer.InitChain,
          request_deserializer=vendor__pb2.RequestInitChain.FromString,
          response_serializer=vendor__pb2.ResponseInitChain.SerializeToString,
      ),
      'BeginBlock': grpc.unary_unary_rpc_method_handler(
          servicer.BeginBlock,
          request_deserializer=vendor__pb2.RequestBeginBlock.FromString,
          response_serializer=vendor__pb2.ResponseBeginBlock.SerializeToString,
      ),
      'EndBlock': grpc.unary_unary_rpc_method_handler(
          servicer.EndBlock,
          request_deserializer=vendor__pb2.RequestEndBlock.FromString,
          response_serializer=vendor__pb2.ResponseEndBlock.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'abci_vendor.ABCIApplication', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import logging
logging.basicConfig(level=logging.DEBUG)

from event_chain.application import app
from event_chain import ec
from event_chain.db import utils
from event_chain.application import models
from event_chain.utils import helpers
